package org.xbib.content.json.jackson;

import com.fasterxml.jackson.databind.JsonNode;

import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.io.StringReader;

/**
 * Utility class to load JSON values from various sources as {@link com.fasterxml.jackson.databind.JsonNode}s.
 * This class uses a {@link JsonNodeReader} to parse JSON inputs.
 *
 * @see JsonNodeReader
 */
public final class JsonLoader {
    /**
     * The reader.
     */
    private static final JsonNodeReader READER = new JsonNodeReader();

    private JsonLoader() {
    }

    /**
     * Read a {@link com.fasterxml.jackson.databind.JsonNode} from an input stream.
     * This method throws an {@link java.io.IOException} if the resource does not
     * exist.
     *
     * @param inputStream the input stream
     * @return the JSON document at the resource
     * @throws IllegalArgumentException resource path does not begin with a
     *                                  {@code /}
     * @throws java.io.IOException      there was a problem loading the resource, or the JSON
     *                                  document is invalid
     */
    public static JsonNode fromResource(InputStream inputStream)
            throws IOException {
        final JsonNode ret;
        try {
            if (inputStream == null) {
                throw new IOException("not found");
            }
            ret = READER.fromInputStream(inputStream);
        } finally {
            if (inputStream != null) {
                inputStream.close();
            }
        }
        return ret;
    }

    /**
     * Read a {@link com.fasterxml.jackson.databind.JsonNode} from a user supplied {@link java.io.Reader}.
     *
     * @param reader The reader
     * @return the document
     * @throws java.io.IOException if the reader has problems
     */
    public static JsonNode fromReader(final Reader reader)
            throws IOException {
        return READER.fromReader(reader);
    }

    /**
     * Read a {@link com.fasterxml.jackson.databind.JsonNode} from a string input.
     *
     * @param json the JSON as a string
     * @return the document
     * @throws java.io.IOException could not read from string
     */
    public static JsonNode fromString(final String json)
            throws IOException {
        return fromReader(new StringReader(json));
    }
}
