package org.xbib.content.rdf.test.io.xml;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.xbib.content.rdf.RdfContentFactory.turtleBuilder;

import org.junit.jupiter.api.Test;
import org.xbib.content.rdf.RdfContentBuilder;
import org.xbib.content.rdf.Resource;
import org.xbib.content.rdf.Triple;
import org.xbib.content.rdf.internal.DefaultAnonymousResource;
import org.xbib.content.rdf.internal.DefaultResource;
import org.xbib.content.rdf.io.ntriple.NTripleContent;
import org.xbib.content.rdf.io.ntriple.NTripleContentParams;
import org.xbib.content.rdf.io.turtle.TurtleContentParams;
import org.xbib.content.rdf.io.xml.AbstractXmlHandler;
import org.xbib.content.rdf.io.xml.AbstractXmlResourceHandler;
import org.xbib.content.rdf.io.xml.XmlContentParser;
import org.xbib.content.rdf.io.xml.XmlHandler;
import org.xbib.content.resource.IRI;
import org.xbib.content.resource.IRINamespaceContext;
import org.xbib.content.resource.NamespaceContext;
import org.xbib.content.rdf.test.StreamTester;
import org.xbib.net.PercentEncoders;

import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

public class XmlReaderTest extends StreamTester {

    @Test
    public void testOAIDC() throws Exception {
        String filename = "oro-eprint-25656.xml";
        try (InputStream inputStream = getClass().getResourceAsStream(filename)) {
            if (inputStream == null) {
                throw new IOException("file " + filename + " not found");
            }
            IRINamespaceContext namespaceContext = IRINamespaceContext.newInstance(getClass());
            namespaceContext.addNamespace("oaidc", "http://www.openarchives.org/OAI/2.0/oai_dc/");
            namespaceContext.addNamespace("dc", "http://purl.org/dc/elements/1.1/");
            TurtleContentParams turtleParams = new TurtleContentParams(namespaceContext, true);
            XmlHandler<TurtleContentParams> turtleHandler = new AbstractXmlResourceHandler<>(turtleParams) {

                @Override
                public boolean isResourceDelimiter(QName name) {
                    return "oai_dc".equals(name.getLocalPart());
                }

                @Override
                public void identify(QName name, String value, IRI identifier) {
                    if ("identifier".equals(name.getLocalPart()) && DefaultResource.isBlank(getResource())) {
                        try {
                            // make sure we can build an opaque IRI, whatever is out there
                            getResource().setId(IRI.create("id:" +
                                    PercentEncoders.getRegNameEncoder(StandardCharsets.UTF_8).encode(value)));
                        } catch (IOException e) {
                            // swallow
                        }
                    }
                }

                @Override
                public boolean skip(QName name) {
                    // skip dc:dc element
                    return "dc".equals(name.getLocalPart());
                }

                @Override
                public XmlHandler<TurtleContentParams> setNamespaceContext(NamespaceContext namespaceContext) {
                    return this;
                }

                @Override
                public IRINamespaceContext getNamespaceContext() {
                    return namespaceContext;
                }
            };
            RdfContentBuilder<TurtleContentParams> builder = turtleBuilder(turtleParams);
            turtleHandler.setBuilder(builder);
            XmlContentParser<TurtleContentParams> parser = new XmlContentParser<>(turtleParams, inputStream);
            parser.setHandler(turtleHandler).parse();
            assertStream("dc.ttl", getClass().getResource("dc.ttl").openStream(),
                    builder.streamInput());
        }
    }

    @Test
    public void testXmlArray() throws Exception {
        String filename = "array.xml";
        try (InputStream inputStream = getClass().getResourceAsStream(filename)) {
            if (inputStream == null) {
                throw new IOException("file " + filename + " not found");
            }
            IRINamespaceContext namespaceContext = IRINamespaceContext.newInstance(getClass());
            NTripleContentParams nTripleContentParams = new NTripleContentParams(namespaceContext);
            AbstractXmlResourceHandler<NTripleContentParams> xmlHandler = new AbstractXmlResourceHandler<>(nTripleContentParams) {

                @Override
                public boolean isResourceDelimiter(QName name) {
                    return false;
                }

                @Override
                public void identify(QName name, String value, IRI identifier) {
                    getResource().setId(IRI.create("id:1"));
                }

                @Override
                public boolean skip(QName name) {
                    return false;
                }

                @Override
                public AbstractXmlResourceHandler<NTripleContentParams> setNamespaceContext(NamespaceContext namespaceContext) {
                    return this;
                }

                @Override
                public IRINamespaceContext getNamespaceContext() {
                    return namespaceContext;
                }
            };

            MyBuilder builder = new MyBuilder();
            xmlHandler.setDefaultNamespace("xml", "http://xmltest")
                    .setBuilder(builder);
            DefaultAnonymousResource.reset();
            XmlContentParser<NTripleContentParams> parser = new XmlContentParser<>(xmlHandler.getParams(), inputStream);
            parser.setHandler(xmlHandler).parse();
            assertEquals("[id:1 xml:dates _:b2, _:b2 xml:date 2001, _:b2 xml:date 2002, _:b2 xml:date 2003]",
                    builder.getTriples().toString()
            );
        }
    }

    @Test
    public void testXmlAttribute() throws Exception {
        String filename = "attribute.xml";
        try (InputStream inputStream = getClass().getResourceAsStream(filename)) {
            if (inputStream == null) {
                throw new IOException("file " + filename + " not found");
            }
            IRINamespaceContext namespaceContext = IRINamespaceContext.newInstance(getClass());
            NTripleContentParams params = new NTripleContentParams(namespaceContext);
            AbstractXmlHandler<NTripleContentParams> xmlHandler = new AbstractXmlResourceHandler<>(params) {
                @Override
                public boolean isResourceDelimiter(QName name) {
                    return false;
                }

                @Override
                public void identify(QName name, String value, IRI identifier) {
                    getResource().setId(IRI.create("id:1"));
                }

                @Override
                public boolean skip(QName name) {
                    return false;
                }

                @Override
                public XmlHandler<NTripleContentParams> setNamespaceContext(NamespaceContext namespaceContext) {
                    return this;
                }

                @Override
                public IRINamespaceContext getNamespaceContext() {
                    return namespaceContext;
                }
            };
            MyBuilder builder = new MyBuilder();
            xmlHandler.setDefaultNamespace("xml", "http://localhost")
                    .setBuilder(builder);
            DefaultAnonymousResource.reset();
            XmlContentParser<NTripleContentParams> parser = new XmlContentParser<>(xmlHandler.getParams(), inputStream);
            parser.setHandler(xmlHandler).parse();
            assertEquals("[id:1 xml:dates _:b2, _:b2 xml:date _:b3, _:b3 xml:@href 1, _:b2 xml:date _:b5, "
                            + "_:b5 xml:@href 2, _:b2 xml:date _:b7, _:b7 xml:@href 3, _:b2 xml:date _:b9, _:b9 xml:hello World]",
                    builder.getTriples().toString()
            );
        }
    }

    private static class MyBuilder extends RdfContentBuilder<NTripleContentParams> {

        private final List<Triple> triples = new LinkedList<>();

        MyBuilder() throws IOException {
            super(NTripleContent.nTripleContent(), new NTripleContentParams(IRINamespaceContext.newInstance(MyBuilder.class)));
        }

        @Override
        public MyBuilder receive(Triple triple) {
            triples.add(triple);
            return this;
        }

        @Override
        public MyBuilder receive(Resource resource) throws IOException {
            triples.addAll(resource.triples());
            return this;
        }

        List<Triple> getTriples() {
            return triples;
        }
    }

}
